@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009-2012 Roger Light <roger@atchoo.org>
+Copyright (c) 2009-2013 Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -53,6 +53,7 @@ struct userdata {
 	char *password;
 	int verbose;
 	bool quiet;
+	bool no_retain;
 };
 
 void my_message_callback(struct mosquitto *mosq, void *obj, const struct mosquitto_message *message)
@@ -62,6 +63,8 @@ void my_message_callback(struct mosquitto *mosq, void *obj, const struct mosquit
 	assert(obj);
 	ud = (struct userdata *)obj;
 
+	if(message->retain && ud->no_retain) return;
+
 	if(ud->verbose){
 		if(message->payloadlen){
 			printf("%s %s\n", message->topic, (const char *)message->payload);
@@ -123,18 +126,21 @@ void print_usage(void)
 	mosquitto_lib_version(&major, &minor, &revision);
 	printf("mosquitto_sub is a simple mqtt client that will subscribe to a single topic and print all messages it receives.\n");
 	printf("mosquitto_sub version %s running on libmosquitto %d.%d.%d.\n\n", VERSION, major, minor, revision);
-	printf("Usage: mosquitto_sub [-c] [-h host] [-k keepalive] [-p port] [-q qos] [-v] -t topic ...\n");
+	printf("Usage: mosquitto_sub [-c] [-h host] [-k keepalive] [-p port] [-q qos] [-R] [-v] -t topic ...\n");
+	printf("                     [-A bind_address]\n");
 	printf("                     [-i id] [-I id_prefix]\n");
 	printf("                     [-d] [--quiet]\n");
 	printf("                     [-u username [-P password]]\n");
 	printf("                     [--will-topic [--will-payload payload] [--will-qos qos] [--will-retain]]\n");
 #ifdef WITH_TLS
-	printf("                     [{--cafile file | --capath dir} [--cert file] [--key file]]\n");
+	printf("                     [{--cafile file | --capath dir} [--cert file] [--key file] [--insecure]]\n");
 #ifdef WITH_TLS_PSK
 	printf("                     [--psk hex-key --psk-identity identity]\n");
 #endif
 #endif
 	printf("       mosquitto_sub --help\n\n");
+	printf(" -A : bind the outgoing socket to this host/ip address. Use to control which interface\n");
+	printf("      the client communicates over.\n");
 	printf(" -c : disable 'clean session' (store subscription and pending messages when client disconnects).\n");
 	printf(" -d : enable debug messages.\n");
 	printf(" -h : mqtt host to connect to. Defaults to localhost.\n");
@@ -144,6 +150,7 @@ void print_usage(void)
 	printf(" -k : keep alive in seconds for this client. Defaults to 60.\n");
 	printf(" -p : network port to connect to. Defaults to 1883.\n");
 	printf(" -q : quality of service level to use for the subscription. Defaults to 0.\n");
+	printf(" -R : do not print stale messages (those with retain set).\n");
 	printf(" -t : mqtt topic to subscribe to. May be repeated multiple times.\n");
 	printf(" -u : provide a username (requires MQTT 3.1 broker)\n");
 	printf(" -v : print published messages verbosely.\n");
@@ -163,6 +170,12 @@ void print_usage(void)
 	printf("            communication.\n");
 	printf(" --cert : client certificate for authentication, if required by server.\n");
 	printf(" --key : client private key for authentication, if required by server.\n");
+	printf(" --tls-version : TLS protocol version, can be one of tlsv1.2 tlsv1.1 or tlsv1.\n");
+	printf("                 Defaults to tlsv1.2 if available.\n");
+	printf(" --insecure : do not check that the server certificate hostname matches the remote\n");
+	printf("              hostname. Using this option means that you cannot be sure that the\n");
+	printf("              remote host is the server you wish to connect to and so is insecure.\n");
+	printf("              Do not use this option in a production environment.\n");
 #ifdef WITH_TLS_PSK
 	printf(" --psk : pre-shared-key in hexadecimal (no leading 0x) to enable TLS-PSK mode.\n");
 	printf(" --psk-identity : client identity string for TLS-PSK mode.\n");
@@ -178,6 +191,7 @@ int main(int argc, char *argv[])
 	int i;
 	char *host = "localhost";
 	int port = 1883;
+	char *bind_address = NULL;
 	int keepalive = 60;
 	bool clean_session = true;
 	bool debug = false;
@@ -194,10 +208,12 @@ int main(int argc, char *argv[])
 	bool will_retain = false;
 	char *will_topic = NULL;
 
+	bool insecure = false;
 	char *cafile = NULL;
 	char *capath = NULL;
 	char *certfile = NULL;
 	char *keyfile = NULL;
+	char *tls_version = NULL;
 
 	char *psk = NULL;
 	char *psk_identity = NULL;
@@ -219,6 +235,15 @@ int main(int argc, char *argv[])
 				}
 			}
 			i++;
+		}else if(!strcmp(argv[i], "-A")){
+			if(i==argc-1){
+				fprintf(stderr, "Error: -A argument given but no address specified.\n\n");
+				print_usage();
+				return 1;
+			}else{
+				bind_address = argv[i+1];
+			}
+			i++;
 		}else if(!strcmp(argv[i], "-c") || !strcmp(argv[i], "--disable-clean-session")){
 			clean_session = false;
 		}else if(!strcmp(argv[i], "--cafile")){
@@ -262,6 +287,8 @@ int main(int argc, char *argv[])
 				host = argv[i+1];
 			}
 			i++;
+		}else if(!strcmp(argv[i], "--insecure")){
+			insecure = true;
 		}else if(!strcmp(argv[i], "-i") || !strcmp(argv[i], "--id")){
 			if(id_prefix){
 				fprintf(stderr, "Error: -i and -I argument cannot be used together.\n\n");
@@ -347,6 +374,8 @@ int main(int argc, char *argv[])
 			i++;
 		}else if(!strcmp(argv[i], "--quiet")){
 			ud.quiet = true;
+		}else if(!strcmp(argv[i], "-R")){
+			ud.no_retain = true;
 		}else if(!strcmp(argv[i], "-t") || !strcmp(argv[i], "--topic")){
 			if(i==argc-1){
 				fprintf(stderr, "Error: -t argument given but no topic specified.\n\n");
@@ -358,6 +387,15 @@ int main(int argc, char *argv[])
 				ud.topics[ud.topic_count-1] = argv[i+1];
 			}
 			i++;
+		}else if(!strcmp(argv[i], "--tls-version")){
+			if(i==argc-1){
+				fprintf(stderr, "Error: --tls-version argument given but no version specified.\n\n");
+				print_usage();
+				return 1;
+			}else{
+				tls_version = argv[i+1];
+			}
+			i++;
 		}else if(!strcmp(argv[i], "-u") || !strcmp(argv[i], "--username")){
 			if(i==argc-1){
 				fprintf(stderr, "Error: -u argument given but no username specified.\n\n");
@@ -515,18 +553,28 @@ int main(int argc, char *argv[])
 		mosquitto_lib_cleanup();
 		return 1;
 	}
+	if(insecure && mosquitto_tls_insecure_set(mosq, true)){
+		if(!ud.quiet) fprintf(stderr, "Error: Problem setting TLS insecure option.\n");
+		mosquitto_lib_cleanup();
+		return 1;
+	}
 	if(psk && mosquitto_tls_psk_set(mosq, psk, psk_identity, NULL)){
 		if(!ud.quiet) fprintf(stderr, "Error: Problem setting TLS-PSK options.\n");
 		mosquitto_lib_cleanup();
 		return 1;
 	}
+	if(tls_version && mosquitto_tls_opts_set(mosq, 1, tls_version, NULL)){
+		if(!ud.quiet) fprintf(stderr, "Error: Problem setting TLS options.\n");
+		mosquitto_lib_cleanup();
+		return 1;
+	}
 	mosquitto_connect_callback_set(mosq, my_connect_callback);
 	mosquitto_message_callback_set(mosq, my_message_callback);
 	if(debug){
 		mosquitto_subscribe_callback_set(mosq, my_subscribe_callback);
 	}
 
-	rc = mosquitto_connect(mosq, host, port, keepalive);
+	rc = mosquitto_connect_bind(mosq, host, port, keepalive, bind_address);
 	if(rc){
 		if(!ud.quiet){
 			if(rc == MOSQ_ERR_ERRNO){
